//---------------------------------------------------------------------------
//~Author: Fabio Arciniegas A.
//~Description
//
//Version $Id$
//$Log$
//~See Licence.html for distribution information
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//Header Includes
//---------------------------------------------------------------------------
#include "ConstructTreeVisitor.h"


ConstructTreeVisitor::ConstructTreeVisitor()
{
      
  try
  {
    XMLPlatformUtils::Initialize();
  }

  catch(const XMLException& toCatch)
  {
    CString error;
    error.Format("%s\n%s\n%s",
		 "Error during Xerces-c Initialization.\n",
		 "Exception message:",
		 XTOC(toCatch.getMessage()));
    AfxMessageBox(error,MB_OK);
  }
}


    /** 
      Create a DOM tree for the xml file and Visit it, creating a
      visual representation on the MFC CTreeCtrl.
      @param xmlFile The structural source (a DOM tree)
      @param visualtree The visual destiny (an MFC tree)
      @return a hashtable with the visual nodes as keys and 
              the structural nodes as values or null if fails
   */

map<HTREEITEM,DOM_Node>
 ConstructTreeVisitor::constructTree(const char * xmlFile,
		                         CTreeCtrl *  mfcTree)
{

  DOMParser *parser;
  bool errorsOccured = false;
  CString error;
  try
  {
    parser = new DOMParser;
    parser->setDoNamespaces(true);
    parser->parse(xmlFile);
  }

  catch (const XMLException& e)
  {
    error.Format("%s\n%s",
		 "An error occured during parsing\n   Message: ",
		 XTOC(e.getMessage()));
    errorsOccured = true;
  }

 
  catch (const DOM_DOMException)
  {
    error.Format("%s \n",
		 "A DOM error occured during parsing\n");
    errorsOccured = true;
  }
      

  catch (...)
  {
    error.Format("%s\n%s",
		 "An error occured during parsing\n   Message: ",
		 "Undetermined");
    errorsOccured = true;
  }

  if(errorsOccured)
  {
    AfxMessageBox(error,MB_OK);
    //return ;//NULL;
  }

  DOM_Node doc = parser->getDocument();
  DOM_NodeList aList = doc.getChildNodes();
  DOM_Node aNode;
  int i = 0;
  // Find the first node element and present it.
  do
  {
	 aNode  = aList.item(i++);
  } while (aNode.getNodeType() != DOM_Node::ELEMENT_NODE);
  // Now, simply call the construction method with the root element
  constructTree(aNode,mfcTree,NULL);
  return visual;
  
}

  /** 
      Visit the DOM tree, creating a visual representation on the
      MFC CTreeCtrl.
      @param domNode The structural source (a DOM tree)
      @param mfctree The visual destiny (an MFC tree)
      @param parent the node to which the results must be appended to
      @return a hashtable with the visual nodes as keys and 
              the structural nodes as values or null if fails
   */

void
ConstructTreeVisitor::constructTree(DOM_Node &   domNode,
		                         CTreeCtrl *  mfcTree,
					 HTREEITEM * parent = NULL)
{
  HTREEITEM current;
  char * text = NULL;
  DOM_NamedNodeMap atts;
  char *attribute = 0; // in case this is an attribute (see case below)
  unsigned int atti; // just an arbitrary index for the attributes
  switch(domNode.getNodeType())
  {
  case DOM_Node::ELEMENT_NODE:
		current = mfcTree->InsertItem(
		    ((DOM_Element &)domNode).getTagName().transcode(),
			0,3,parent == NULL ? TVI_ROOT: *parent);
		atts = domNode.getAttributes();
		visual[current] = domNode;
  		for(atti = 0; atti < atts.getLength() ; atti++)
		  constructTree(atts.item(atti),mfcTree,&current); 
		break;
  case DOM_Node::TEXT_NODE:
	  text = ((DOM_Text &)domNode).getNodeValue().transcode();
	  text = trim(text);
	  if(strlen(text))
	  {
		current = mfcTree->InsertItem(text,
			1,3,parent == NULL ? TVI_ROOT: *parent);
    	visual[current] = domNode;
  	  }
	  delete[] text;
	  break;
  case DOM_Node::ATTRIBUTE_NODE:
	   attribute = new char[((DOM_Attr &)domNode).getName().length() +
					  ((DOM_Attr &)domNode).getValue().length() +
					  2];
	   strcpy(attribute,((DOM_Attr &)domNode).getName().transcode());
	   strcat(attribute,"=");
	   strcat(attribute,((DOM_Attr &)domNode).getValue().transcode());
	   current = mfcTree->InsertItem(attribute,2,3,
		                             parent == NULL ? TVI_ROOT: *parent);
	   visual[current] = domNode;
  	   delete []attribute;
		break;
  default:
	  return;
  }
  if(domNode.getNodeType() == DOM_Node::ATTRIBUTE_NODE)
	  return; // the descendants of the node (its value) were already treated
  DOM_NodeList aList = domNode.getChildNodes();
  for(unsigned int i = 0; i < aList.getLength(); i++)
    constructTree(aList.item(i),mfcTree,&current);
}

 
  /** 
      delete leading and trailing whitespaces
      @param source the original 
      @return another string without trailing or leading whitespaces
   */
char *trim(char *source)
{
  int i,j,h;
  char * result = new char[strlen(source)+1];
  for(i=0; source[i] == '\n' ||
	   source[i] == ' '  ||
	   source[i] == '\t';i++);
  
  for(j=strlen(source)-1; source[j] == '\n' ||
	                  source[j] == ' '  ||
	                  source[j] == '\t';j--);
  for(h = 0;i <= j;i++)
    result[h++] = source[i];
  result[h] = 0;
  return result;
}







