#ifndef ConstructTreeVisitor_H
#define ConstructTreeVisitor_H

//---------------------------------------------------------------------------
// Includes and Constants
//---------------------------------------------------------------------------
#include <util/PlatformUtils.hpp>
#include <util/XMLString.hpp>
#include <util/XMLUniDefs.hpp>
#include <util/TranscodingException.hpp>
#include <framework/XMLFormatter.hpp>
#include <dom/DOM_DOMException.hpp>
#include <dom/DOM.hpp>
#include <parsers/DOMParser.hpp>

#include <string.h>
#include <stdlib.h>
#include <map>

#include "stdafx.h"



#ifndef XTOC
#define XTOC(x) XMLString::transcode(x) // Use iff x is a char *
#define CTOX(x) XMLString::transcode(x) // Use iff x is an XMLCh *
#endif

using namespace std;

//---------------------------------------------------------------------------
// Class definition
//---------------------------------------------------------------------------
/**
   Constructs an MFC representation of a DOM tree.
   Upon construction, it returns a hashtable with the bindings 
   Visual Nodes vs. DOM Nodes, so the client can perform direct
   modifications in the future.

   @author Fabio Arciniegas A.
   @version $Id$
 */

class ConstructTreeVisitor {
 public:
  /**@name Constructors
     Simply start the xerces xml engine and get ready to receive
     parsing requests.
   */
  //@{
  ConstructTreeVisitor();
  //@}
 
  /**@name Visitor Methods
   */
  //@{

    /** 
      Create a DOM tree for the xml file and Visit it, creating a
      visual representation on the MFC CTreeCtrl.
      @param xmlFile The structural source (a DOM tree)
      @param visualtree The visual destiny (an MFC tree)
      @return a hashtable with the visual nodes as keys and 
              the structural nodes as values or null if fails
   */

	map<HTREEITEM ,DOM_Node >
	constructTree(const char * xmlFile,
		     CTreeCtrl *  visualtree);
  //@}

  /** 
      Visit the DOM tree, creating a visual representation on the
      MFC CTreeCtrl.
      @param domtree The structural source (a DOM tree)
      @param visualtree The visual destiny (an MFC tree)
      @return a hashtable with the visual nodes as keys and 
              the structural nodes as values or null if fails
   */
 
  void constructTree(DOM_Node &   domtree,
		     CTreeCtrl *  mfcTree,
                     HTREEITEM * parent);

 private:
  //visual to structural map
  map<HTREEITEM ,DOM_Node > visual;
};

// Helper Functions

  /** 
      delete leading and trailing whitespaces
      @param source the original 
      @return another string without trailing or leading whitespaces
   */
char * trim(char *source);


#endif ConstructTreeVisitor_H
 
