//---------------------------------------------------------------------------
//~Author: Fabio Arciniegas A.
//~Description    
//   StatisticsVisitor is a visitor class for DOM trees which implements the
//   most common statistics questions about a node, namely: get its name as
//   string, count the number of characters that it has (plus those of its
//   children) and the number of children it has.
//
//Version $Id$
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//Header Includes
//---------------------------------------------------------------------------
#include "StatisticsVisitor.h"

//---------------------------------------------------------------------------
// Constructors / Destructors
//---------------------------------------------------------------------------

/**@name Constructor Methods
 */
//@{

StatisticsVisitor::StatisticsVisitor(DOM_Node &dom) : myNode(dom)
{
}

//@}

//---------------------------------------------------------------------------
// Analizers
//---------------------------------------------------------------------------
/**@name Statistic Methods
*/
//@{

char *StatisticsVisitor::getNodeTypeAsString()
{
  switch(myNode.getNodeType())
  {
  case DOM_Node::TEXT_NODE:
    return "Text";

  case DOM_Node::ELEMENT_NODE:
    return "Element";

  case DOM_Node::ATTRIBUTE_NODE:
    return "Attribute";

  case DOM_Node::ENTITY_REFERENCE_NODE:
    return "Entity Reference";

  case DOM_Node::CDATA_SECTION_NODE:
    return "CDATA";

  case DOM_Node::COMMENT_NODE:
    return "Comment";

  case DOM_Node::DOCUMENT_TYPE_NODE:
    return "Document Type";

  case DOM_Node::ENTITY_NODE:
    return "Entity";

  case DOM_Node::XML_DECL_NODE:
    return "XML Declearation";
  default:
    return "Unknown"; //never happens. Included to avoid compilation warnings
  }
}


unsigned int   StatisticsVisitor::getChildrenCount(DOM_Node *node)
{
  if(node == 0)
	node = &myNode;
  unsigned int result = 0;
  DOM_NodeList aList = node->getChildNodes();
  result = aList.getLength();
  for(unsigned int i = 0; i < aList.getLength() ; i++)
    result += getChildrenCount(&aList.item(i));
  return result;
}

//@}







