// xmlquickviewerDlg.cpp : implementation file
//

#include "stdafx.h"
#include "xmlquickviewer.h"
#include "xmlquickviewerDlg.h"
#include "MyAboutDialog.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CXmlquickviewerDlg dialog

CXmlquickviewerDlg::CXmlquickviewerDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CXmlquickviewerDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CXmlquickviewerDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CXmlquickviewerDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CXmlquickviewerDlg)
	DDX_Control(pDX, IDC_NAMESPACE_LI, m_namespaceLI);
	DDX_Control(pDX, IDC_UNAME_ED, m_unameED);
	DDX_Control(pDX, IDC_CHILD_ED, m_ChildED);
	DDX_Control(pDX, IDC_TypeED, m_TypeED);
	DDX_Control(pDX, IDC_FILE_ED, m_FileED);
	DDX_Control(pDX, IDC_BROWSE_BT, m_BrowseBT);
	DDX_Control(pDX, IDC_DOM_TR, m_DOMTree);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CXmlquickviewerDlg, CDialog)
	//{{AFX_MSG_MAP(CXmlquickviewerDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BROWSE_BT, OnBrowseBtClick)
	ON_NOTIFY(TVN_SELCHANGED, IDC_DOM_TR, OnSelchangedDomTr)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_HELP_ABOUT, OnHelpAbout)
	ON_COMMAND(ID_FILE_EXIT, OnFileExit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CXmlquickviewerDlg message handlers

BOOL CXmlquickviewerDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	m_images.Create(16,16,ILC_COLOR,2,2);
	m_images.Add(AfxGetApp()->LoadIcon(IDI_ROOT_IC));
	m_images.Add(AfxGetApp()->LoadIcon(IDI_ELEMENT_IC));
	m_images.Add(AfxGetApp()->LoadIcon(IDI_TEXT_IC));
	m_images.Add(AfxGetApp()->LoadIcon(IDI_OPEN_IC));	
	m_DOMTree.SetImageList(&m_images,TVSIL_NORMAL);
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CXmlquickviewerDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
		CDialog::OnSysCommand(nID, lParam);

}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CXmlquickviewerDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CXmlquickviewerDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CXmlquickviewerDlg::OnBrowseBtClick() 
{

	CFileDialog dlg(TRUE,"xml", "*.xml");
	ConstructTreeVisitor *ctv;
	ctv = new ConstructTreeVisitor();
	m_DOMTree.DeleteAllItems();
	if(dlg.DoModal() == IDOK)
	{
		m_FileED.SetWindowText(dlg.GetPathName().GetBuffer(25));
		visual = ctv->constructTree(dlg.GetPathName().GetBuffer(25),&m_DOMTree);
	}
}

void CXmlquickviewerDlg::OnSelchangedDomTr(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	*pResult = 0;
	HTREEITEM selected = pNMTreeView->itemNew.hItem;
	map<HTREEITEM,DOM_Node>::iterator result;
	DOM_Node myNode;
	if((result = visual.find(selected)) != visual.end())
	{
		myNode = (*result).second;
		StatisticsVisitor *sv = new StatisticsVisitor(myNode);

		// Set the node type name
		m_TypeED.SetWindowText(sv->getNodeTypeAsString());
		
		// Set the children count
		char temp[6];
		m_ChildED.SetWindowText(itoa(sv->getChildrenCount(),temp,10));

		NamespacesVisitor *nv = new NamespacesVisitor(myNode);
		// Set the Qualified Name
		m_unameED.SetWindowText(nv->getQualifiedName());

		// Show the visible namespaces
		vector<char *> nms = nv->getVisibleNamespaces();
		m_namespaceLI.ResetContent();
		while(nms.size() > 0)
		{
			m_namespaceLI.InsertString(0,nms.back());
			nms.pop_back();
		}
		
	}
}

void CXmlquickviewerDlg::OnFileOpen() 
{
	
	OnBrowseBtClick();
}

void CXmlquickviewerDlg::OnHelpAbout() 
{
	MyAboutDialog a;
	a.DoModal();	
}

void CXmlquickviewerDlg::OnFileExit() 
{
	OnOK();
	
}
